/*
-----------------------------------------------------------------------------
This source file is part of OGRE
    (Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/

Copyright (c) 2000-2005 The OGRE Team
Also see acknowledgements in Readme.html

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/
#pragma once
#include "Vector3.h"
#include <memory.h>

// NB All code adapted from Wild Magic 0.2 Matrix math (free source code)
// http://www.geometrictools.com/

// NOTE.  The (x,y,z) coordinate system is assumed to be right-handed.
// Coordinate axis rotation matrices are of the form
//   RX =    1       0       0
//           0     cos(t) -sin(t)
//           0     sin(t)  cos(t)
// where t > 0 indicates a counterclockwise rotation in the yz-plane
//   RY =  cos(t)    0     sin(t)
//           0       1       0
//        -sin(t)    0     cos(t)
// where t > 0 indicates a counterclockwise rotation in the zx-plane
//   RZ =  cos(t) -sin(t)    0
//         sin(t)  cos(t)    0
//           0       0       1
// where t > 0 indicates a counterclockwise rotation in the xy-plane.

/** A 3x3 matrix which can represent rotations around axes.
    @note
        <b>All the code is adapted from the Wild Magic 0.2 Matrix
        library (http://www.geometrictools.com/).</b>
    @par
        The coordinate system is assumed to be <b>right-handed</b>.
*/

class Matrix3x3;
void MultiplyWithTranspose(Matrix3x3 &dest, const Vector3 &a, const Vector3& b);
Matrix3x3 MultiplyWithTranspose(const Vector3 &a, const Vector3& b);

class Matrix3x3
{
public:
    /** Default constructor.
        @note
            It does <b>NOT</b> initialize the matrix for efficiency.
    */
	inline Matrix3x3 () {};
    inline explicit Matrix3x3 (const double arr[3][3])
	{
		memcpy(m,arr,9*sizeof(double));
	}
    inline Matrix3x3 (const Matrix3x3& rkMatrix)
	{
		memcpy(m,rkMatrix.m,9*sizeof(double));
	}
    Matrix3x3 (double fEntry00, double fEntry01, double fEntry02,
                double fEntry10, double fEntry11, double fEntry12,
                double fEntry20, double fEntry21, double fEntry22)
	{
		m[0][0] = fEntry00;
		m[0][1] = fEntry01;
		m[0][2] = fEntry02;
		m[1][0] = fEntry10;
		m[1][1] = fEntry11;
		m[1][2] = fEntry12;
		m[2][0] = fEntry20;
		m[2][1] = fEntry21;
		m[2][2] = fEntry22;
	}

    // member access, allows use of construct mat[r][c]
    inline double* operator[] (size_t iRow) const
	{
		return (double*)m[iRow];
	}
    /*inline operator double* ()
	{
		return (double*)m[0];
	}*/
    Vector3 GetColumn (size_t iCol) const;
    void SetColumn(size_t iCol, const Vector3& vec);
    void FromAxes(const Vector3& xAxis, const Vector3& yAxis, const Vector3& zAxis);

    // assignment and comparison
    inline Matrix3x3& operator= (const Matrix3x3& rkMatrix)
	{
		memcpy(m,rkMatrix.m,9*sizeof(double));
		return *this;
	}
    bool operator== (const Matrix3x3& rkMatrix) const;
    inline bool operator!= (const Matrix3x3& rkMatrix) const
	{
		return !operator==(rkMatrix);
	}

    // arithmetic operations
    Matrix3x3 operator+ (const Matrix3x3& rkMatrix) const;
    Matrix3x3 &operator+= (const Matrix3x3& rkMatrix);
    Matrix3x3 operator- (const Matrix3x3& rkMatrix) const;
    Matrix3x3 operator* (const Matrix3x3& rkMatrix) const;
    Matrix3x3 operator- () const;

    // matrix * vector [3x3 * 3x1 = 3x1]
    Vector3 operator* (const Vector3& rkVector) const;

    // vector * matrix [1x3 * 3x3 = 1x3]
    friend Vector3 operator* (const Vector3& rkVector,
        const Matrix3x3& rkMatrix);

    // matrix * scalar
    Matrix3x3 operator* (double fScalar) const;

    // scalar * matrix
    friend Matrix3x3 operator* (double fScalar, const Matrix3x3& rkMatrix);

    // utilities
    Matrix3x3 Transpose () const;
    bool Inverse (Matrix3x3& rkInverse, double fTolerance = 1e-06) const;
    Matrix3x3 Inverse (double fTolerance = 1e-06) const;
    double Determinant () const;

    // singular value decomposition
    void SingularValueDecomposition (Matrix3x3& rkL, Vector3& rkS,
        Matrix3x3& rkR) const;
    void SingularValueComposition (const Matrix3x3& rkL,
        const Vector3& rkS, const Matrix3x3& rkR);

    // Gram-Schmidt orthonormalization (applied to columns of rotation matrix)
    void Orthonormalize ();

    // orthogonal Q, diagonal D, upper triangular U stored as (u01,u02,u12)
    void QDUDecomposition (Matrix3x3& rkQ, Vector3& rkD,
        Vector3& rkU) const;

    double SpectralNorm () const;
    // eigensolver, matrix must be symmetric
    void EigenSolveSymmetric (double afEigenvalue[3],
        Vector3 akEigenvector[3]) const;

    static void TensorProduct (const Vector3& rkU, const Vector3& rkV,
        Matrix3x3& rkProduct);

    static const double EPSILON;
    static const Matrix3x3 ZERO;
    static const Matrix3x3 IDENTITY;

	friend void MultiplyWithTranspose(Matrix3x3 &dest, const Vector3 &a, const Vector3& b);

    double m[3][3];
protected:
    // support for eigensolver
    void Tridiagonal (double afDiag[3], double afSubDiag[3]);
    bool QLAlgorithm (double afDiag[3], double afSubDiag[3]);

    // support for singular value decomposition
    static const double ms_fSvdEpsilon;
    static const unsigned int ms_iSvdMaxIterations;
    static void Bidiagonalize (Matrix3x3& kA, Matrix3x3& kL,
        Matrix3x3& kR);
    static void GolubKahanStep (Matrix3x3& kA, Matrix3x3& kL,
        Matrix3x3& kR);

    // support for spectral norm
    static double MaxCubicRoot (double afCoeff[3]);

};
