#pragma once
#include <Math.h>
#include <assert.h>

class Vector2f
{
public:
    union {
        struct {
            float x, y;
        };
        float val[2];
    };

public:
    inline Vector2f()
		: x( 0 ), y( 0 )
    {
    }

    inline Vector2f( const float fX, const float fY )
        : x( fX ), y( fY )
    {
    }

    inline explicit Vector2f( const float afCoordinate[2] )
        : x( afCoordinate[0] ),
          y( afCoordinate[1] )
    {
    }

    //inline explicit Vector2f( const int afCoordinate[3] )
    //{
    //    x = (float)afCoordinate[0];
    //    y = (float)afCoordinate[1];
    //    z = (float)afCoordinate[2];
    //}

    inline explicit Vector2f( float* const r )
        : x( r[0] ), y( r[1] )
    {
    }

    inline explicit Vector2f( const float scalar )
        : x( scalar )
        , y( scalar )
    {
    }


    inline Vector2f( const Vector2f& rkVector )
        : x( rkVector.x ), y( rkVector.y )
    {
    }

	inline float operator [] ( const size_t i ) const
    {
        assert( i < 2 );

        return *(&x+i);
    }

	inline float& operator [] ( const size_t i )
    {
        assert( i < 2 );

        return *(&x+i);
    }

    /** Assigns the value of the other vector.
        @param
            rkVector The other vector
    */
    inline Vector2f& operator = ( const Vector2f& rkVector )
    {
        x = rkVector.x;
        y = rkVector.y;

        return *this;
    }

    inline Vector2f& operator = ( const float fScalar )
    {
        x = fScalar;
        y = fScalar;

        return *this;
    }

    inline bool operator == ( const Vector2f& rkVector ) const
    {
        return ( x == rkVector.x && y == rkVector.y );
    }

    inline bool operator != ( const Vector2f& rkVector ) const
    {
        return ( x != rkVector.x || y != rkVector.y );
    }

    // arithmetic operations
    inline Vector2f operator + ( const Vector2f& rkVector ) const
    {
        Vector2f kSum;

        kSum.x = x + rkVector.x;
        kSum.y = y + rkVector.y;

        return kSum;
    }

    inline Vector2f operator - ( const Vector2f& rkVector ) const
    {
        Vector2f kDiff;

        kDiff.x = x - rkVector.x;
        kDiff.y = y - rkVector.y;

        return kDiff;
    }

    inline Vector2f operator * ( const float fScalar ) const
    {
        Vector2f kProd;

        kProd.x = fScalar*x;
        kProd.y = fScalar*y;

        return kProd;
    }

    inline Vector2f operator * ( const Vector2f& rhs) const
    {
        Vector2f kProd;

        kProd.x = rhs.x * x;
        kProd.y = rhs.y * y;

        return kProd;
    }

    inline Vector2f operator / ( const float fScalar ) const
    {
        assert( fScalar != 0.0f );

        Vector2f kDiv;

        float fInv = 1.0f / fScalar;
        kDiv.x = x * fInv;
        kDiv.y = y * fInv;

        return kDiv;
    }

    inline Vector2f operator / ( const Vector2f& rhs) const
    {
        Vector2f kDiv;

        kDiv.x = x / rhs.x;
        kDiv.y = y / rhs.y;

        return kDiv;
    }


    inline Vector2f operator - () const
    {
        Vector2f kNeg;

        kNeg.x = -x;
        kNeg.y = -y;
        return kNeg;
    }

    // overloaded operators to help Vector2f
    inline friend Vector2f operator * ( const float fScalar, const Vector2f& rkVector )
    {
        Vector2f kProd;

        kProd.x = fScalar * rkVector.x;
        kProd.y = fScalar * rkVector.y;

        return kProd;
    }

    inline friend Vector2f operator + (const Vector2f& lhs, const float rhs)
    {
        Vector2f ret(rhs);
        return ret += lhs;
    }

    inline friend Vector2f operator + (const float lhs, const Vector2f& rhs)
    {
        Vector2f ret(lhs);
        return ret += rhs;
    }

    inline friend Vector2f operator - (const Vector2f& lhs, const float rhs)
    {
        return lhs - Vector2f(rhs);
    }

    inline friend Vector2f operator - (const float lhs, const Vector2f& rhs)
    {
        Vector2f ret(lhs);
        return ret -= rhs;
    }

    // arithmetic updates
    inline Vector2f& operator += ( const Vector2f& rkVector )
    {
        x += rkVector.x;
        y += rkVector.y;

        return *this;
    }

    inline Vector2f& operator += ( const float fScalar )
    {
        x += fScalar;
        y += fScalar;
        return *this;
    }

    inline Vector2f& operator -= ( const Vector2f& rkVector )
    {
        x -= rkVector.x;
        y -= rkVector.y;

        return *this;
    }

    inline Vector2f& operator -= ( const float fScalar )
    {
        x -= fScalar;
        y -= fScalar;
        return *this;
    }

    inline Vector2f& operator *= ( const float fScalar )
    {
        x *= fScalar;
        y *= fScalar;
        return *this;
    }

    inline Vector2f& operator *= ( const Vector2f& rkVector )
    {
        x *= rkVector.x;
        y *= rkVector.y;

        return *this;
    }

    inline Vector2f& operator /= ( const float fScalar )
    {
        assert( fScalar != 0.0f );

        float fInv = 1.0f / fScalar;

        x *= fInv;
        y *= fInv;

        return *this;
    }

    inline Vector2f& operator /= ( const Vector2f& rkVector )
    {
        x /= rkVector.x;
        y /= rkVector.y;

        return *this;
    }


    /** Returns the length (magnitude) of the vector.
        @warning
            This operation requires a square root and is expensive in
            terms of CPU operations. If you don't need to know the exact
            length (e.g. for just comparing lengths) use squaredLength()
            instead.
    */
    inline float Length () const
    {
        return sqrt( x * x + y * y );
    }
    inline float Magnitude () const
    {
        return sqrt( x * x + y * y );
    }
    /** Returns the square of the length(magnitude) of the vector.
        @remarks
            This  method is for efficiency - calculating the actual
            length of a vector requires a square root, which is expensive
            in terms of the operations required. This method returns the
            square of the length of the vector, i.e. the same as the
            length but before the square root is taken. Use this if you
            want to find the longest / shortest vector without incurring
            the square root.
    */
    inline float SquaredLength () const
    {
        return x * x + y * y;
    }

    /** Calculates the dot (scalar) product of this vector with another.
        @remarks
            The dot product can be used to calculate the angle between 2
            vectors. If both are unit vectors, the dot product is the
            cosine of the angle; otherwise the dot product must be
            divided by the product of the lengths of both vectors to get
            the cosine of the angle. This result can further be used to
            calculate the distance of a point from a plane.
        @param
            vec Vector with which to calculate the dot product (together
            with this one).
        @returns
            A float representing the dot product value.
    */
    inline float DotProduct(const Vector2f& vec) const
    {
        return x * vec.x + y * vec.y;
    }

    inline float AngleBetween(const Vector2f& vec) const
    {
		float dot = DotProduct(vec);
		float len = Length() + vec.Length();
		float cos = dot / len;
		float angle = acos(cos);
        return angle;
    }

    /** Normalizes the vector.
        @remarks
            This method Normalizes the vector such that it's
            length / magnitude is 1. The result is called a unit vector.
        @note
            This function will not crash for zero-sized vectors, but there
            will be no changes made to their components.
        @returns The previous length of the vector.
    */
    inline float Normalize()
    {
        float fLength = sqrt( x * x + y * y );

        // Will also work for zero-sized vectors, but will change nothing
        if ( fLength > 1e-08 )
        {
            float fInvLength = 1.0f / fLength;
            x *= fInvLength;
            y *= fInvLength;
        }

        return fLength;
    }

    /** Calculates the cross-product of 2 vectors, i.e. the vector that
        lies perpendicular to them both.
        @remarks
            The cross-product is normally used to calculate the normal
            vector of a plane, by calculating the cross-product of 2
            non-equivalent vectors which lie on the plane (e.g. 2 edges
            of a triangle).
        @param
            vec Vector which, together with this one, will be used to
            calculate the cross-product.
        @returns
            A vector which is the result of the cross-product. This
            vector will <b>NOT</b> be Normalized, to maximise efficiency
            - call Vector2f::Normalize on the result if you wish this to
            be done. As for which side the resultant vector will be on, the
            returned vector will be on the side from which the arc from 'this'
            to rkVector is anticlockwise, e.g. UNIT_Y.crossProduct(UNIT_Z)
            = UNIT_X, whilst UNIT_Z.crossProduct(UNIT_Y) = -UNIT_X.
			This is because OGRE uses a right-handed coordinate system.
        @par
            For a clearer explanation, look a the left and the bottom edges
            of your monitor's screen. Assume that the first vector is the
            left edge and the second vector is the bottom edge, both of
            them starting from the lower-left corner of the screen. The
            resulting vector is going to be perpendicular to both of them
            and will go <i>inside</i> the screen, towards the cathode tube
            (assuming you're using a CRT monitor, of course).
    */
    inline float CrossProduct( const Vector2f& rkVector ) const
    {
        return x * rkVector.y - y * rkVector.x;
    }

    /** Returns true if the vector's scalar components are all greater
        that the ones of the vector it is compared against.
    */
    inline bool operator < ( const Vector2f& rhs ) const
    {
        if( x < rhs.x && y < rhs.y )
            return true;
        return false;
    }

    /** Returns true if the vector's scalar components are all smaller
        that the ones of the vector it is compared against.
    */
    inline bool operator > ( const Vector2f& rhs ) const
    {
        if( x > rhs.x && y > rhs.y )
            return true;
        return false;
    }

    /** As Normalize, except that this vector is unaffected and the
        Normalized vector is returned as a copy. */
    inline Vector2f NormalizedCopy(void) const
    {
        Vector2f ret = *this;
        ret.Normalize();
        return ret;
    }

    /** Calculates a reflection vector to the plane with the given normal .
    @remarks NB assumes 'this' is pointing AWAY FROM the plane, invert if it is not.
    */
    inline Vector2f Reflect(const Vector2f& normal) const
    {
        return Vector2f( *this - ( 2 * this->DotProduct(normal) * normal ) );
    }
	
    static const Vector2f ZERO;
    static const Vector2f UNIT_X;
    static const Vector2f UNIT_Y;
};
