#include "stdafx.h"
#include "FastLSM.h"
#include <queue>
#include <limits.h>
#include <functional>
#include <float.h>
#include "jacobi.h"
#include <assert.h>
#include <string>
#include "InvSqrt.h"
#include "FastLSM.h"

void Body::ShapeMatch()
{
	// Recalculate the various invaraints (mainly region properties dependent on the particles) if necessary
	if(invariantsDirty)
	{
		CalculateInvariants();
		invariantsDirty = false;
	}

	// Set each particle's sumData in preparation for calculating F(mixi) and F(mixi0T)
	for each(Particle *particle in particles)
	{
		particle->sumData.v = particle->perRegionMass * particle->x;
		particle->sumData.M = MultiplyWithTranspose(particle->perRegionMass * particle->x, particle->x0);
	}

	// Calculate F(mixi) and F(mixi0T)
	SumParticlesToRegions();

	// Shape match
	for each(Region *r in regions)
	{
		Vector3f Fmixi = r->sumData.v;
		Matrix3x3f Fmixi0T = r->sumData.M;

		r->c = (1 / r->M) * Fmixi;										// Eqn. 9
		r->A = Fmixi0T - MultiplyWithTranspose(r->M * r->c, r->c0);		// Enq. 11

		// ### Polar decompose Ar following the method in Mueller et al. 2005:

		// Multiply by the stored eigenvectors (for faster polar decomposition - see Extensions section of FastLSM paper)
		Matrix3x3f S = r->A.Transpose() * r->A;
		S = r->eigenVectors.Transpose() * S * r->eigenVectors;
		float eigenValues[3];
		jacobi(3, (float*)S.m, (float*)eigenValues, (float*)r->eigenVectors.m);

		for(int j = 0; j < 3; j++)
		{
			if(eigenValues[j] <= 0)
			{
				eigenValues[j] = 0.05f;
			}
			eigenValues[j] = InvSqrt(eigenValues[j]);
		}

		Matrix3x3f DPrime;
		DPrime = Matrix3x3f(eigenValues[0], 0, 0, 0, eigenValues[1], 0, 0, 0, eigenValues[2]);
		S = r->eigenVectors * DPrime * r->eigenVectors.Transpose();
		
		// Now we can get the rotation part
		r->R = r->A * S;

		// ### Done with the polar decomposition

		// Test for inversion (flipping of the rest configuration)
		// Disabled for fracturing objects as it can cause some screwups with degenerate (planar, linear) regions
		if(r->R.Determinant() < 0 && fracturing == false)
		{
			r->R *= -1;
		}

		// Set the translation part
		r->t = r->c - r->R * r->c0;

		// Set the region's SumData in preparation for calculating F(Tr)
		r->sumData.M = r->R;
		r->sumData.v = r->t;
	}

	// Calculate F(Tr)
	SumRegionsToParticles();

	// Calculate goal positions for the particles
	for each(Particle *particle in particles)
	{
		float invNumParentRegions = 1.0f / particle->parentRegions.size();

		// Eqn. 12, split into rotation and translation
		particle->g = (invNumParentRegions * particle->sumData.M) * particle->x0 + invNumParentRegions * particle->sumData.v;

		// Store just the rotational part too; it's useful for rendering
		particle->R = invNumParentRegions * particle->sumData.M;
	}
}

Matrix3x3f MrMatrix(Vector3f v)
{
	Matrix3x3f ret;
	ret.m[0][0] = (v.z * v.z + v.y * v.y);
	ret.m[0][1] = (-v.x*v.y);
	ret.m[0][2] = (-v.x*v.z);
	ret.m[1][1] = (v.z*v.z + v.x*v.x);
	ret.m[1][2] = (-v.z*v.y);
	ret.m[2][2] = (v.y*v.y + v.x*v.x);
	ret.m[1][0] = ret.m[0][1];
	ret.m[2][0] = ret.m[0][2];
	ret.m[2][1] = ret.m[1][2];
	return ret;
}

// Damp velocity per region according to the method of Mueller et al. 2006 (Position Based Dynamics), optimized for LSM as described in the paper
void Body::PerformRegionDamping()
{
	if(kRegionDamping == 0.0)
		return;

	// Set the data needed to calculate F(mivi), F(mix~ivi) and F(mix~ix~iT)
	for each(Particle *particle in particles)
	{
		// This is for F(mivi)
		particle->sumData.v = particle->perRegionMass * particle->v;

		// This is for F(mix~ivi)
		particle->sumData.M.SetColumn(0, particle->x.CrossProduct(particle->sumData.v));

		// This is for F(mix~ix~iT)
		// We take advantage of the fact that this is a symmetric matrix to squeeze the data in the standard SumData
		Vector3f x = particle->x;
		particle->sumData.M.m[2][1] = particle->perRegionMass * (x.z * x.z + x.y * x.y);
		particle->sumData.M.m[0][1] = particle->perRegionMass * (-x.x*x.y);
		particle->sumData.M.m[0][2] = particle->perRegionMass * (-x.x*x.z);
		particle->sumData.M.m[1][1] = particle->perRegionMass * (x.z*x.z + x.x*x.x);
		particle->sumData.M.m[1][2] = particle->perRegionMass * (-x.z*x.y);
		particle->sumData.M.m[2][2] = particle->perRegionMass * (x.y*x.y + x.x*x.x);
	}

	SumParticlesToRegions();

	for each(Region *region in regions)
	{
		Vector3f v = Vector3f::ZERO;
		Vector3f L = Vector3f::ZERO;
		Matrix3x3f I = Matrix3x3f::ZERO;

		// Rebuild the original symmetric matrix from the reduced data
		Matrix3x3f &M = region->sumData.M;
		Matrix3x3f FmixixiT = Matrix3x3f(M.m[2][1], M.m[0][1], M.m[0][2], M.m[0][1], M.m[1][1], M.m[1][2], M.m[0][2], M.m[1][2], M.m[2][2]);

		// Calculate v, L, I, w
		v = (1.0f / region->M) * region->sumData.v;							// Eqn. 14
		L = M.GetColumn(0) - region->c.CrossProduct(region->sumData.v);
		I = FmixixiT - region->M * MrMatrix(region->c);

		Vector3f w;
		w = I.Inverse() * L;

		// Set the data needed to apply this to the particles
		region->sumData.v = v;
		region->sumData.M.SetColumn(0, w);
		region->sumData.M.SetColumn(1, w.CrossProduct(region->c));
	}

	SumRegionsToParticles();

	// Apply calculated damping
	for each(Particle *particle in particles)
	{
		if(particle->parentRegions.size() > 0)
		{
			Vector3f Fv = particle->sumData.v;
			Vector3f Fw = particle->sumData.M.GetColumn(0);
			Vector3f Fwc = particle->sumData.M.GetColumn(1);
			Vector3f dv = (1.0f / particle->parentRegions.size()) * (Fv + Fw.CrossProduct(particle->x) - Fwc - (float)particle->parentRegions.size() * particle->v);
			
			// Bleed off non-rigid motion
			particle->v = particle->v + kRegionDamping * dv;
		}
	}
}

void Body::CalculateParticleVelocities(float h)
{
	for each(Particle *particle in particles)
	{
		if(particle->parentRegions.size() == 0)
		{
			// We are just a lone particle flying about - no regions, so no goal position - so only account for fExt
			particle->v = particle->v + h * (particle->f / particle->mass);

			// Set the goal in case we want to render particle goals
			particle->g = particle->x;
		}
		else
		{
			// We have a goal position
			particle->v = particle->v + alpha * (particle->g - particle->x) / h + h * (particle->f / particle->mass);	// Eqn. 1
		}
		particle->f = Vector3f::ZERO;			// Zero the force accumulator
	}
}

void Body::ApplyParticleVelocities(float h)
{
	for each(Particle *particle in particles)
	{
		particle->x = particle->x + h * particle->v;		// Eqn. 2
	}
}

void Body::UpdateCellPositions()
{
	for each(Cell *cell in cells)
	{
		cell->UpdateVertexPositions();
	}
}

void Body::SumParticlesToRegions()
{
	for each(Summation *bar in sums[0])
	{
		bar->SumFromChildren();
	}
	for each(Summation *plate in sums[1])
	{
		plate->SumFromChildren();
	}
	for each(Region *region in regions)
	{
		region->SumFromChildren();
	}
}

void Body::SumRegionsToParticles()
{
	for each(Summation *plate in sums[1])
	{
		plate->SumFromParents();
	}
	for each(Summation *bar in sums[0])
	{
		bar->SumFromParents();
	}
	for each(Particle *particle in particles)
	{
		particle->SumFromParents();
	}
}
