#include "Testbed.h"
#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <ctime>

std::vector<Body*> Testbed::bodies;
int Testbed::frame = 0;

void Testbed::Initialize()
{
	srand((unsigned int)time(NULL));
}

void Testbed::AddBody()
{
	Body *body = new Body(Vector3f(1.0f,1.0f,1.0f));
	body->w = rand() % 2 + 1;
	body->alpha = 0.75f;
	body->fracturing = true;
	body->fractureDistanceTolerance = 999.0f;
	body->fractureRotationTolerance = 0.6f;
	body->kRegionDamping = 0.25f;
	bodies.push_back(body);
	
	int width = rand() % 3 + 2;
	int height = rand() % 10 + 2;
	int depth = rand() % 3 + 2;

	for(int x = 0; x < width; x++)
	{
		for(int y = 0; y < height; y++)
		{
			for(int z = 0; z < depth; z++)
			{
				body->AddParticle(Point3(x,y,z));
			}
		}
	}

	body->Finalize();
	
	// Add a position offset and a random velocity
	const float velocityMax = 25;
	const float velocityHalf = velocityMax / 2;
	Vector3f randomVelocity = Vector3f((rand() % 1000 / 1000.0f) * velocityMax - velocityHalf, (rand() % 1000 / 1000.0f) * velocityMax - velocityHalf, (rand() % 1000 / 1000.0f) * velocityMax - velocityHalf);
	for each(Particle *particle in body->particles)
	{
		particle->x.y += 5;
		particle->v += randomVelocity;
	}
}

void Testbed::Update()
{
	const int MAX_BODIES = 50;
	const int SPAWN_RATE = 30;
	const int TARGET_FPS = 30;
	const float h = 1.0f / TARGET_FPS;

	if(frame % SPAWN_RATE == 0 && bodies.size() < MAX_BODIES)
		AddBody();
	frame++;

	for each(Body *body in bodies)
	{
		// Do FastLSM simulation
		body->ShapeMatch();
		body->CalculateParticleVelocities(h);
		body->PerformRegionDamping();
		body->ApplyParticleVelocities(h);
		body->DoFracturing();

		// Apply gravity and check floor
		const Vector3f gravity = Vector3f(0, -9.8f, 0);

		for each(Particle *particle in body->particles)
		{
			particle->f += gravity;

			if(particle->x.y < 0)
			{
				// This particle has hit the floor
				particle->f.y -= particle->x.y;
				particle->v = Vector3f::ZERO;
				particle->x.y = 0;
			}
		}
	}
}

void glVertex(Vector3f v)
{
	glVertex3f(v.x, v.y, v.z);
}

void glNormal(Vector3f v)
{
	glNormal3f(v.x, v.y, v.z);
}

void Testbed::Render()
{
	// Position the camera
	gluLookAt(0, 20, 60, 0, 0, 0, 0, 1, 0);

	// Draw the floor
	glDisable(GL_LIGHTING);
	glColor4f(0.5f, 0.5f, 0.5f, 1.0f);
	glBegin(GL_QUADS);
	glVertex3f(-9999, -100, -9999);
	glVertex3f(-9999, -100, 9999);
	glVertex3f(9999, -100, 9999);
	glVertex3f(9999, -100, -9999);
	glEnd();

	// Draw the cells
	glEnable(GL_LIGHTING);
	glEnable(GL_NORMALIZE);

	glMaterialf(GL_FRONT, GL_SHININESS, 28);
	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);

	glColor4f(0.753f, 0.764f, 0.910f, 1);

	// Draw the quads of the cells
	glBegin(GL_QUADS);
	for each(Body *body in bodies)
	{
		body->UpdateCellPositions();

		for each(Cell *c in body->cells)
		{
			if(c->center->edge)
			{
				Vector3f p[8];
				p[0] = c->vertices[7].position;
				p[1] = c->vertices[6].position;
				p[2] = c->vertices[5].position;
				p[3] = c->vertices[4].position;
				p[4] = c->vertices[3].position;
				p[5] = c->vertices[2].position;
				p[6] = c->vertices[1].position;
				p[7] = c->vertices[0].position;

				// Bottom face
				glNormal((p[3]-p[7]).CrossProduct(p[6]-p[7]));

				glVertex(p[7]);
				glVertex(p[3]);
				glVertex(p[2]);
				glVertex(p[6]);

				// Top face
				glNormal((p[4]-p[5]).CrossProduct(p[1]-p[5]));

				glVertex(p[5]);
				glVertex(p[4]);
				glVertex(p[0]);
				glVertex(p[1]);

				// Far face
				glNormal((p[5]-p[7]).CrossProduct(p[3]-p[7]));

				glVertex(p[7]);
				glVertex(p[5]);
				glVertex(p[1]);
				glVertex(p[3]);


				// Right face
				glNormal((p[1]-p[3]).CrossProduct(p[2]-p[3]));

				glVertex(p[3]); 
				glVertex(p[1]);
				glVertex(p[0]);
				glVertex(p[2]);


				// Front face
				glNormal((p[2]-p[6]).CrossProduct(p[4]-p[6]));

				glVertex(p[6]);
				glVertex(p[2]);
				glVertex(p[0]);
				glVertex(p[4]);

				// Left Face
				glNormal((p[6]-p[7]).CrossProduct(p[5]-p[7]));

				glVertex(p[7]);
				glVertex(p[6]);
				glVertex(p[4]);
				glVertex(p[5]);
			}
		}
	}
	glEnd();
		
	// Render the lines around the cubes
	glColor3f(0.0f, 0.0f, 0.0f);
	
	glLineWidth(2.0f);
	glDisable(GL_LIGHTING);
	glBegin(GL_LINES);
	for each(Body *body in bodies)
	{
		for each(Cell *c in body->cells)
		{
			if(c->center->edge)
			{
				Vector3f p[8];
				p[0] = c->vertices[7].position;
				p[1] = c->vertices[6].position;
				p[2] = c->vertices[5].position;
				p[3] = c->vertices[4].position;
				p[4] = c->vertices[3].position;
				p[5] = c->vertices[2].position;
				p[6] = c->vertices[1].position;
				p[7] = c->vertices[0].position;

				// Bottom face.
				glVertex(p[7]);
				glVertex(p[3]);
				glVertex(p[3]);
				glVertex(p[2]);
				glVertex(p[2]);
				glVertex(p[6]);
				glVertex(p[6]);
				glVertex(p[7]);

				// Top face.
				glVertex(p[5]);
				glVertex(p[4]);
				glVertex(p[4]);
				glVertex(p[0]);
				glVertex(p[0]);
				glVertex(p[1]);
				glVertex(p[1]);
				glVertex(p[5]);

				// Far face.
				glVertex(p[7]);
				glVertex(p[5]);
				glVertex(p[5]);
				glVertex(p[1]);
				glVertex(p[1]);
				glVertex(p[3]);
				glVertex(p[3]);
				glVertex(p[7]);

				// Right face.
				glVertex(p[3]); 
				glVertex(p[1]);
				glVertex(p[1]);
				glVertex(p[0]);
				glVertex(p[0]);
				glVertex(p[2]);
				glVertex(p[2]);
				glVertex(p[3]);

				// Front face.
				glVertex(p[6]);
				glVertex(p[2]);
				glVertex(p[2]);
				glVertex(p[0]);
				glVertex(p[0]);
				glVertex(p[4]);
				glVertex(p[4]);
				glVertex(p[6]);

				// Left Face.
				glVertex(p[7]);
				glVertex(p[6]);
				glVertex(p[6]);
				glVertex(p[4]);
				glVertex(p[4]);
				glVertex(p[5]);
				glVertex(p[5]);
				glVertex(p[7]);
			}
		}
	}
	glEnd();
	glEnable(GL_LIGHTING);
}
